﻿/*
 * Example connection to the viking.trade public web socket API
 */

using System;
using System.Threading;
using System.Threading.Tasks;
using Websocket.Client; //nuget package
using Free.Ports.zLib; //To decompress incoming messages. Source code included under zlib.net folder. This code is from the excellent https://github.com/JoshDullen/zlib.net

namespace viking_trade
{
    class Program
    {
        private const int OutBufSize = 32768;
        static async Task Main(string[] args)
        {
            //init zlib for incoming messages decompression
            var zs = new zlib.z_stream();
            if (zlib.Z_OK != zlib.inflateInit2(zs, -15)) {
                Console.WriteLine("Error. Zlib init failed. Exiting");
                System.Environment.Exit(0);
            }
            zs.out_buf = new byte[OutBufSize];
            zs.avail_out = OutBufSize;
            zs.next_out = 0;
            zs.total_out = 0;

            var exitEvent = new ManualResetEvent(false);
            var url = new Uri("wss://api.viking.trade/public");
            
            using (var client = new WebsocketClient(url))
            {
                client.ReconnectionHappened.Subscribe(type => {
                    Console.WriteLine($"Connected to: {url}");
                    
                    client.Send("{\"P\":\"g_sym\"}"); //requesting instruments list
                    
                    var heartbeatTimer = new System.Timers.Timer();
                    heartbeatTimer.Interval = 5000;
                    heartbeatTimer.Elapsed += delegate (Object source, System.Timers.ElapsedEventArgs e) {
                        client.Send("7"); //heartbeat each 5 seconds
                    };
                    heartbeatTimer.Enabled = true;
                });

                //Msg handler
                client.MessageReceived.Subscribe( msg => {
                    String text = null;
                    if (msg.MessageType == System.Net.WebSockets.WebSocketMessageType.Text) {
                        text = msg.Text;
                    } else
                    if (msg.MessageType == System.Net.WebSockets.WebSocketMessageType.Binary) {//decompress
                        var tail = new byte[4] { 0x00, 0x00, 0xFF, 0xFF };
                        var rv = new byte[msg.Binary.Length + tail.Length];
                        System.Buffer.BlockCopy(msg.Binary, 0, rv, 0, msg.Binary.Length);
                        System.Buffer.BlockCopy(tail, 0, rv, msg.Binary.Length, tail.Length);
                        zs.next_in = 0;
                        zs.avail_in = (uint) rv.Length;
                        zs.in_buf = rv;
                        zs.avail_out = OutBufSize;
                        zs.next_out = 0;
                        zlib.inflate(zs, zlib.Z_SYNC_FLUSH);
                        var res = new byte[zs.next_out];
                        System.Buffer.BlockCopy(zs.out_buf, 0, res, 0, zs.next_out);
                        text = System.Text.Encoding.Default.GetString(res);
                    } else
                    if (msg.MessageType == System.Net.WebSockets.WebSocketMessageType.Close) {
                        return;
                    }
                    Console.WriteLine($"Recieved: {text}");
		    //Your message handler...
                });

                await client.Start();

                exitEvent.WaitOne();
            }
        }
    }
}
